
import bpy
import importlib

from . import constants, properties, operators, utils
from .importer import importer
from .ui import panels, topbar
from .mdx_parser import parser
from .classes import class_storage

bl_info = {
    'name': 'Warcraft 3 (*.mdx)',
    'author': 'ProgFZ',
    'version': (0, 0, 1),
    'blender': (2, 92, 0),
    'category': 'Import-Export',
    'location': 'File > Import',
    'description': 'Import *.mdx files (3d models of WarCraft 3)',
    'doc_url': 'do not exist',
    'tracker_url': 'do not exist'
    }






modules = [
    constants,
    operators,
    utils,
    class_storage,
    parser,
    importer,
    panels,
    topbar
   
]

classes = [
    operators.WarCraft3OperatorImportMDX,
    operators.FixWar3Character,
    topbar.TOPBAR_MT_FixWar3
    
]


def register():
    """
    This function registers the addon classes when the addon is enabled.
    """
    # reload the submodules
    for module in modules:
        importlib.reload(module)

    # register the properties
    properties.register()
    class_storage.register()
    parser.register()
    importer.register()

    # add a function to the event timer that will fire after the addon is enabled
    #bpy.app.timers.register(utilities.addon_enabled)

    # add an event handler that will save and load  preferences from the blend file
    #bpy.app.handlers.load_post.append(utilities.setup_project)
    #bpy.app.handlers.load_post.append(utilities.load_properties)
    #bpy.app.handlers.save_pre.append(utilities.save_properties)

    # register the classes
    for cls in classes:
        bpy.utils.register_class(cls)
    
    bpy.types.TOPBAR_MT_file_import.append(operators.menu_import_mdx)
    topbar.add_pipeline_menu()


def unregister():
    """
    This function unregisters the addon classes when the addon is disabled.
    """
    # remove the pipeline menu
    #header_menu.remove_parent_menu()
    topbar.remove_parent_menu()    

    bpy.types.TOPBAR_MT_file_import.remove(operators.menu_import_mdx)

    # unregister the classes
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

    # remove event handlers
    #bpy.app.handlers.load_post.remove(utilities.setup_project)
    #bpy.app.handlers.load_post.remove(utilities.load_properties)
    #bpy.app.handlers.save_pre.remove(utilities.save_properties)


    importer.unregister()
    parser.unregister()
    class_storage.unregister()
    properties.unregister()