from array import array
from typing import List, Tuple


class WarCraft3Attachment:
    def __init__(self):
        self.type = 'attachment'
        self.node = None
        self.path = None
        self.attachment_id = None
        self.visibility = None


class WarCraft3Bone:
    def __init__(self):
        self.type = 'bone'
        self.node = None
        self.geoset_id:int = None
        self.geoset_animation_id:int = None

class WarCraft3CollisionShape:
    def __init__(self):
        self.type = 'collision_shape'
        self.shape = None
        self.node = None
        self.points = None
        self.radius = None

class WarCraft3Event:
    def __init__(self):
        self.type = 'event'
        self.node = None
        self.global_sequence_id = None
        self.keys = None
        self.timeline = None

class WarCraft3GeosetAnimation:
    def __init__(self):
        self.geoset_id = None
        self.animation_color = None
        self.animation_alpha = None
        self.flags = 0

class WarCraft3GeosetTransformation:
    def __init__(self):
        self.tracks_count = None
        self.interpolation_type = None
        self.global_sequence_id = None
        self.times = []
        self.values = []
        self.in_tans = []
        self.out_tans = []

class WarCraft3Helper:
    def __init__(self):
        self.type = 'helper'
        self.node = None

class WarCraft3Layer:
    def __init__(self):
        self.filterMode = None
        self.shadingFlags = None
        self.texture_id = None
        self.textureAnimationId = None
        self.coordId = None
        self.alpha = None
        self.material_alpha = None
        self.material_texture_id = None
        self.emissive_gain = None
        self.fresnel_color = None
        self.fresnel_opacity = None
        self.fresnel_team_color = None
        self.emissions = None
        self.fresnel_alpha = None

class WarCraft3Material:
    def __init__(self):
        self.layers = []
        self.hd = False

class WarCraft3MeshGeo:
    def __init__(self):
        self.size = 0
        self.name:str = None
        self.vertices:List[Tuple[float,float,float]] = None
        self.normals:List[Tuple[float,float,float]] = None

        self.tangents:array = None
        self.skin:array = None
        
        self.faceTypeGroups:array = None
        self.faceGroups:array = None
        self.faces:List[Tuple[int,int,int]] = None
        
        self.vertexGroups:array = None
        self.matrixGroups:array = None
        self.matrixIndices:array = None

        self.materialId:int = 0
        self.selectionGroup:int = 0
        self.selectionFlags:int = 0

        self.lod:int = 0
        self.lodName:str = None

        self.extent:Tuple(float,Tuple(float,float,float),Tuple(float,float,float)) = None
        self.extents_count:int = 0
        self.extents:List[Tuple(float,Tuple(float,float,float),Tuple(float,float,float))] = []

        self.uvs:List[List[Tuple[float,float]]] = []


        
class WarCraft3Model:

    def __init__(self):
        self.file:str = ''
        self.version = 800
        self.name:str = None
        self.meshes: List[WarCraft3MeshGeo] = []
        self.materials: List[WarCraft3Material] = []
        self.textures: List[WarCraft3Texture] = []
        # self.nodes: List[]  = []
        self.nodes = []
        self.sequences: List[WarCraft3Sequence] = []
        self.geoset_animations: List[WarCraft3GeosetAnimation] = []
        self.pivot_points: List[float] = []
        self.face_effects: List[FaceFxEffect] = []
        self.bind_pose: List = []
        self.globalSequences:array[int] = None

    def normalize_meshes_names(self):
        for mesh in self.meshes:
            if mesh.lodName and mesh.lodName!="":
                mesh.name = mesh.lodName
            else:
                mesh.name = self.name


class WarCraft3ModelExtent:
    def __init__(self):
        self.boundsRadius = 0
        self.min = (0,0,0)
        self.max = (0,0,0)


class FaceFxEffect:
    def __init__(self):
        self.type:str = ''
        self.path:str = ''

class WarCraft3Node:
    def __init__(self):
        self.name = None
        self.id = None
        self.parent = None
        self.translations = None
        self.rotations = None
        self.scalings = None
        self.flags = None

class WarCraft3Sequence:
    def __init__(self):
        self.name = None
        self.movement_speed = 0
        self.interval_start = None
        self.interval_end = None
        self.flags = 0
        self.rarity = 0
        self.sync_point = 0
        self.bounds_radius = 0
        self.minimum_extent = None
        self.maximum_extent = None

class WarCraft3Texture:
    def __init__(self):
        self.image_file_name = None
        self.replaceable_id:int = None
        self.flags:int = 0
