
from typing import List,Tuple

import bpy
from bpy.types import Material

from ..classes.MdxParts import WarCraft3Model
from ..properties import MDXImportProperties

def flipUV(uv:Tuple[float,float])->Tuple[float,float]:
    return(uv[0],1-uv[1])
    

def create_mesh_objects(model: WarCraft3Model, bpy_materials: List[Material], import_properties: MDXImportProperties):
    scale = import_properties.scale_global
    scaleV = (scale,scale,scale)
    print("creating mesh")
    bpy_objects = []

    for warCraft3Mesh in model.meshes:
        print("creating geoset")
        if warCraft3Mesh.lod>0 and import_properties.clear_LODs:
            continue
        bpy_mesh = bpy.data.meshes.new(warCraft3Mesh.name)
        if import_properties.auto_merge:
            merge = bpy.data.objects.get(warCraft3Mesh.name)
        else:
            merge = None
        bpy_object = bpy.data.objects.new(warCraft3Mesh.name, bpy_mesh)
        bpy.context.scene.collection.objects.link(bpy_object)
        bpy_mesh.from_pydata(warCraft3Mesh.vertices, (), warCraft3Mesh.faces)
        print("mesh loaded")
        bpy_mesh.uv_layers.new()
        uv_layer = bpy_mesh.uv_layers.active.data

        # for loop in bpy_mesh.loops:
        #     loopIndex = loop.index
        #     uv_layer[loopIndex]
        #     uv_layer.data[loop.index].uv = uv_co[loop.index]

        for tris in bpy_mesh.polygons:
            for loopIndex in range(tris.loop_start, tris.loop_start + tris.loop_total):
                vertex_index = bpy_mesh.loops[loopIndex].vertex_index
                uv_layer[loopIndex].uv = (flipUV(warCraft3Mesh.uvs[0][vertex_index]))
        
        print("uvs loaded")
        if not merge:
            bpy_material = bpy_materials[warCraft3Mesh.materialId]
            bpy_mesh.materials.append(bpy_material)
        else:
            bpy_mesh.materials.append(merge.data.materials[0])

        #vertex_groups, vertex_groups_ids = recalculateVertexGroups(warCraft3Mesh.vertexGroups,warCraft3Mesh.matrixGroups,warCraft3Mesh.matrixIndices)

        #for vertexGroupId in vertex_groups:
        #    bpy_object.vertex_groups.new(name=str(vertexGroupId))

        #print("vertex groups loaded")

        vertex_groups = {}
        skin = warCraft3Mesh.skin
        for vertex_index in range(len(skin)//8):
            offset = vertex_index*8
            #skin_weight = warCraft3Mesh.skin[vertex_index]
            for i in range(4):
                x=offset+i
                if skin[x] != 255 and skin[4+x] != 0:
                    vg = vertex_groups.get(skin[x])
                    if not vg:
                        vg = ([],[])
                        vertex_groups[skin[x]] = vg
                    vg[0].append(vertex_index)
                    vg[1].append(skin[4+x]/255.0)
        for k,vg in vertex_groups.items():
            vertg = bpy_object.vertex_groups.new(name=str(k))
            for i in range(len(vg[0])):
                vertg.add([vg[0][i]],vg[1][i],'ADD')
        
        bpy_object.scale = (scaleV)
        # for vertex_index in range(len(warCraft3Mesh.skin)):
        #     skin_weight = warCraft3Mesh.skin[vertex_index]
        #     # if skin_weight[0] != 255 and skin_weight[4] != 0:
        #     #     bpy_object.vertex_groups.get(str(skin_weight[0])).add([vertex_index, ], skin_weight[4]/255.0, 'REPLACE')
        #     # if skin_weight[1] != 255 and skin_weight[5] != 0:
        #     #     bpy_object.vertex_groups.get(str(skin_weight[1])).add([vertex_index, ], skin_weight[5]/255.0, 'REPLACE')
        #     # if skin_weight[2] != 255 and skin_weight[6] != 0:
        #     #     bpy_object.vertex_groups.get(str(skin_weight[2])).add([vertex_index, ], skin_weight[6]/255.0, 'REPLACE')
        #     # if skin_weight[3] != 255 and skin_weight[7] != 0:
        #     #     bpy_object.vertex_groups.get(str(skin_weight[3])).add([vertex_index, ], skin_weight[7]/255.0, 'REPLACE')

        #     # RMS had a bug which filled some list with -1 resulting in them being filled with 255 in the saved models
        #     if sum(skin_weight[4:]) > 300:
        #         for i in range(0, 5):
        #             sw = skin_weight[i + 4]
        #             if sw != 0:
        #                 bpy_object.vertex_groups.get(str(skin_weight[i])).add([vertex_index, ], sw / 255.0, 'REPLACE')

            # if skin_weight[4] != 0:
            #     bpy_object.vertex_groups.get(str(skin_weight[0])).add([vertex_index, ], skin_weight[4]/255.0, 'REPLACE')
            # if skin_weight[5] != 0:
            #     bpy_object.vertex_groups.get(str(skin_weight[1])).add([vertex_index, ], skin_weight[5]/255.0, 'REPLACE')
            # if skin_weight[6] != 0:
            #     bpy_object.vertex_groups.get(str(skin_weight[2])).add([vertex_index, ], skin_weight[6]/255.0, 'REPLACE')
            # if skin_weight[7] != 0:
            #     bpy_object.vertex_groups.get(str(skin_weight[3])).add([vertex_index, ], skin_weight[7]/255.0, 'REPLACE')

        # for vertex_index, vertexGroupIds in enumerate(warCraft3Mesh.vertexGroups):
        #     for vertexGroupId in vertexGroupIds:
        #         bpy_object.vertex_groups.get(str(vertexGroupId)).add([vertex_index, ], 1.0, 'REPLACE')

        if merge:
            print("merge "+merge.name+" "+bpy_object.name)
            ctx = bpy.context.copy()
            ctx['active_object'] = merge
            ctx['selected_editable_objects'] = [merge, bpy_object]
            bpy.ops.object.join(ctx)
        else:
            if warCraft3Mesh.lod>0:
                bpy_object.hide_set(True)
            bpy_objects.append(bpy_object)
    return bpy_objects


def recalculateVertexGroups(matrix_groups, matrix_groups_sizes, matrix_indices):
    i = 0
    matrix = []

    for matrixGroupSize in matrix_groups_sizes:
        matrix.append(matrix_indices[i: i + matrixGroupSize])
        i += matrixGroupSize

    vertex_groups = []
    vertex_groups_ids = set()

    for matrixGroup in matrix_groups:
        if matrixGroup < len(matrix):
            vertex_group = matrix[matrixGroup]
            vertex_groups.append(vertex_group)

            for vertexGroupId in vertex_group:
                vertex_groups_ids.add(vertexGroupId)

    if len(vertex_groups) == 0:
        for m_i in matrix_indices:
            vertex_groups.append([])
            vertex_groups_ids.add(m_i)

    vertex_groups_ids = list(vertex_groups_ids)
    vertex_groups_ids.sort()

    return vertex_groups, vertex_groups_ids