import bpy
import importlib
from typing import List

from bpy.types import Material, Object

from ..properties import MDXImportProperties
from ..classes.MdxParts import WarCraft3Model
from . import (
    create_armature_actions,
    create_armature_object,
    create_mesh_objects,
    create_material,
    create_object_actions,
    fix_character
)



submodules = [
    create_armature_actions,
    create_armature_object,
    create_mesh_objects,
    create_material,
    create_object_actions,
    fix_character
]

classes = [

]

def register():
    for module in submodules:
        importlib.reload(module)

    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)




def load_warcraft_to_ue_model(model: WarCraft3Model, import_properties: MDXImportProperties):

    bpy_materials: List[Material] = create_material.create_material(model, import_properties.team_color, import_properties)
    bpy_objects: List[Object] = create_mesh_objects.create_mesh_objects(model, bpy_materials, import_properties)
    armature_object: Object = create_armature_object.create_armature_object(model, bpy_objects, import_properties)
    create_armature_actions.create_armature_actions(armature_object, model, import_properties.frame_time, import_properties)
    #create_object_actions.create_object_actions(model, bpy_objects, import_properties.frame_time)