from array import ArrayType, array
from io import IOBase, SEEK_CUR
import struct
from typing import List, Tuple


class Reader:
    def __init__(self, data):
        self.__data = data
        self.offset = 0

    def getf(self, value_format):
        size = struct.calcsize(value_format)
        self.offset += size
        return struct.unpack_from(value_format, self.__data, self.offset - size)

    def gets(self, size):
        
        self.offset += size
        return struct.unpack_from('<{}s'.format(size), self.__data, self.offset - size)[0].split(b'\x00')[0].decode(encoding='mbcs')

    def getid(self, required_chunks_id, debug=False):
        self.offset += 4
        chunk_id = struct.unpack_from('<4s', self.__data, self.offset - 4)[0].decode(encoding='mbcs')

        if type(required_chunks_id) == str:
            required_chunks_id = [required_chunks_id, ]

        if chunk_id not in required_chunks_id:
            if not debug:
                raise Exception('chunk id "{0}" not in "{1}"'.format(chunk_id, required_chunks_id))
        else:
            return chunk_id

    def skip(self, count):
        self.offset += count


class StreamReader:
    intsize = struct.calcsize('<I')
    intsize32 = struct.calcsize('<L')
    trianglesize = struct.calcsize('<3H')
    floatsize = struct.calcsize('<f')
    vertexsize = struct.calcsize('<3f')
    vertexsize2 = struct.calcsize('<2f')
    vertexsize4 = struct.calcsize('<4f')

    def __init__(self, stream:IOBase):
        self.__stream = stream
        #self.__intsize = struct.calcsize('<I')
        self.mark:int = 0

    def readStr(self, size:int)->str:
        self.mark += size
        return struct.unpack('<{}s'.format(size), self.__stream.read(size))[0].split(b'\x00')[0].decode(encoding='mbcs')

    def readRaw(self,size:int):
        self.mark += size
        return self.__stream.read(size)

    def readF(self, value_format:str):
        size = struct.calcsize(value_format)
        self.mark += size
        return struct.unpack(value_format, self.__stream.read(size))

    def readInt(self)->int:
        self.mark += StreamReader.intsize
        return struct.unpack('<I', self.__stream.read( StreamReader.intsize))[0]

    def readInt32(self)->int:
        self.mark += StreamReader.intsize32
        return struct.unpack('<L', self.__stream.read( StreamReader.intsize32))[0]

    def readSInt32(self)->int:
        self.mark += StreamReader.intsize32
        return struct.unpack('<l', self.__stream.read( StreamReader.intsize32))[0]

    def readFloat(self)->float:
        self.mark += StreamReader.floatsize
        return struct.unpack('<f', self.__stream.read( StreamReader.floatsize))[0]

    def readVertex(self)->Tuple:
        self.mark += StreamReader.vertexsize
        return struct.unpack('<3f', self.__stream.read( StreamReader.vertexsize))

    def readVertex4(self)->Tuple:
        self.mark += StreamReader.vertexsize4
        return struct.unpack('<4f', self.__stream.read( StreamReader.vertexsize4))

    def read4cc(self):
        self.mark += 4
        raw:bytes = self.__stream.read(4)
        if len(raw)==4:
            return struct.unpack('<4s', raw)[0].decode(encoding='mbcs')
        else:
            return None

    def readArray(self,count:int, typecode:str)->ArrayType:
        
        a = array(typecode)
        a.fromfile(self.__stream,count)
        self.mark += a.itemsize*count
        return a

    def readVertexList(self,count:int) -> List[Tuple[float,float,float]]:
        self.mark += count*StreamReader.vertexsize
        return [x for x in struct.iter_unpack('<3f', self.__stream.read(count*StreamReader.vertexsize))]

    def readVertexList2(self,count:int) -> List[Tuple[float,float]]:
        self.mark += count*StreamReader.vertexsize2
        return [x for x in struct.iter_unpack('<2f', self.__stream.read(count*StreamReader.vertexsize2))]

    def readVertexList4(self,count:int) -> List[Tuple[float,float,float,float]]:
        self.mark += count*StreamReader.vertexsize4
        return [x for x in struct.iter_unpack('<4f', self.__stream.read(count*StreamReader.vertexsize4))]

    def readTriangleList(self,count:int) -> List[Tuple[int,int,int]]:
        self.mark += count*StreamReader.trianglesize
        return [x for x in struct.iter_unpack('<3H', self.__stream.read(count*StreamReader.trianglesize))]


    def readFloatArray(self,count:int):
        a = array('f')
        a.fromfile(self.__stream,count)
        self.mark += a.itemsize*count
        return a
    
    def readUIntArray(self,count:int):
        a = array('L')
        a.fromfile(self.__stream,count)
        self.mark += a.itemsize*count
        return a

    def skip(self, size:int):
        self.mark += size
        self.__stream.seek(size, SEEK_CUR)

    def markReset(self):
        self.mark = 0



