import bpy
from . import utils


class WarCraft3Preferences(bpy.types.AddonPreferences):
    bl_idname = 'warcraft2ue'
    resourceFolder: bpy.props.StringProperty(
        name='Resource',
        default='',
        subtype='DIR_PATH'
        )
    alternativeResourceFolder: bpy.props.StringProperty(
        name='Alternative Resource',
        default='',
        subtype='DIR_PATH'
        )
    textureExtension: bpy.props.StringProperty(
        name='Image Extension',
        default='png'
        )

    def draw(self, context):
        layout = self.layout
        layout.prop(self, 'resourceFolder')
        layout.prop(self, 'alternativeResourceFolder')
        layout.prop(self, 'textureExtension')

class WarCraft3ArmatureSequenceList(bpy.types.PropertyGroup):
    name: bpy.props.StringProperty()


class WarCraft3ArmatureProperties(bpy.types.PropertyGroup):
    bpy_type = bpy.types.Armature
    sequencesList: bpy.props.CollectionProperty(type=WarCraft3ArmatureSequenceList)
    sequencesListIndex: bpy.props.IntProperty(update=utils.set_animation)


class WarCraft3BoneProperties(bpy.types.PropertyGroup):
    bpy_type = bpy.types.Bone
    nodeType: bpy.props.EnumProperty(
        items=[
            ('NONE', 'None', ''),
            ('BONE', 'Bone', ''),
            ('HELPER', 'Helper', ''),
            ('ATTACHMENT', 'Attachment', ''),
            ('EVENT', 'Event', ''),
            ('COLLISION_SHAPE', 'Collision Shape', '')
            ],
        name='Node Type',
        update=utils.set_bone_node_type,
        default='NONE'
        )

class MDXImportProperties:
    def __init__(self):
        self.mdx_file_path: str = ''
        self.team_color = None
        self.bone_size: float = 1.0
        self.use_custom_fps: bool = False
        self.fps: float = 30
        self.frame_time: float = 1.0
        self.axis_order = 'XYZ'
        self.scale_global = 1.0
        self.auto_merge = True
        self.clear_LODs = True

    def calculate_frame_time(self):
        if not self.use_custom_fps:
            self.fps = bpy.context.scene.render.fps
        self.frame_time = 1000 / self.fps


prop_classes = (
    WarCraft3Preferences,
    WarCraft3ArmatureSequenceList,
    WarCraft3ArmatureProperties,
    WarCraft3BoneProperties,
    #MDXImportProperties
)

def register():

    for cls in prop_classes:
        bpy.utils.register_class(cls)

    WarCraft3ArmatureProperties.bpy_type.warcraft_to_ue = bpy.props.PointerProperty(type=WarCraft3ArmatureProperties)
    WarCraft3BoneProperties.bpy_type.warcraft_to_ue = bpy.props.PointerProperty(type=WarCraft3BoneProperties)

def unregister():

    del WarCraft3BoneProperties.bpy_type.warcraft_to_ue
    del WarCraft3ArmatureProperties.bpy_type.warcraft_to_ue

    for cls in reversed(prop_classes):
        bpy.utils.unregister_class(cls)