import bpy


class TOPBAR_MT_FixWar3(bpy.types.Menu):
    """
    This defines a new class that will be the menu, "Utilities".
    """
    bl_idname = "TOPBAR_MT_FixWar3"
    bl_label = "Fix War3 Models"

    def draw(self, context):
        self.layout.operator('wm.fix_war3_character_model')


class TOPBAR_MT_Pipeline(bpy.types.Menu):
    """
    This defines a new class that will be the top most parent menu, "Pipeline".
    All the other action menu items are children of this.
    """
    bl_idname = "TOPBAR_MT_Pipeline"
    bl_label = "Pipeline"

    def draw(self, context):
        pass


def pipeline_menu(self, context):
    """
    This function creates the pipeline menu item. This will be referenced in other functions
    as a means of appending and removing it's contents from the top bar editor class
    definition.

    :param object self: This refers the the Menu class definition that this function will
    be appended to.
    :param object context: This parameter will take the current blender context by default,
    or can be passed an explicit context.
    """
    self.layout.menu(TOPBAR_MT_Pipeline.bl_idname)

def fixes_menu(self, context):
    """
    This function creates the utilities menu item. This will be referenced in other functions
    as a means of appending and removing it's contents from the top bar editor class
    definition.

    :param object self: This refers the the Menu class definition that this function will
    be appended to.
    :param object context: This parameter will take the current blender context by default,
    or can be passed an explicit context.
    """
    self.layout.menu(TOPBAR_MT_FixWar3.bl_idname)


def add_pipeline_menu():
    """
    This function adds the Parent "Pipeline" menu item by appending the pipeline_menu()
    function to the top bar editor class definition.
    """
    if not hasattr(bpy.types, TOPBAR_MT_Pipeline.bl_idname):
        bpy.utils.register_class(TOPBAR_MT_Pipeline)
        bpy.types.TOPBAR_MT_editor_menus.append(pipeline_menu)

    try:
        bpy.types.TOPBAR_MT_Pipeline.remove(fixes_menu)

    finally:
        bpy.types.TOPBAR_MT_Pipeline.append(fixes_menu)

def remove_parent_menu():
    """
    This function removes the Parent "Pipeline" menu item by removing the pipeline_menu()
    function from the top bar editor class definition.
    """
    if hasattr(bpy.types, TOPBAR_MT_Pipeline.bl_idname):
        bpy.utils.unregister_class(TOPBAR_MT_Pipeline)
        bpy.types.TOPBAR_MT_editor_menus.remove(pipeline_menu)
